--
-- Trapper
--

-- This addon won't show up on ACP for some reason unknown

local trapper      		= CreateFrame("Frame","TrapperFrame",UIParent)
local trapper_options 	= CreateFrame("Frame","TrapperOptions")

-- accounting variables

local ver="4.0"

-- trap button information (up to 5) { button, name, number of traps }

local trap_b	={}
local trap_n	={}
local trap_i	={}
local trap_sid	={}
local ntraps

-- saved variables

Trapper_vars =	{ scale }

local Trapper_opts = { scale }
--

--
-- Trapper Options Functions
--

function trapper_init_options()
	trapper_options.name="Trapper"
	trapper_options.okay=trapper_options_ok
	trapper_options.cancel=trapper_options_cancel
	trapper_options:SetScript("OnShow",trapper_options_show)
	
	trapper_options.scaleslide=CreateFrame("Slider","trapperoptionsscale",trapper_options,"OptionsSliderTemplate")
	
	local scaleslide=trapper_options.scaleslide
	
	scaleslide:SetWidth(100)
	scaleslide:SetHeight(16)
	scaleslide:SetPoint("TOPLEFT",trapper_options,20,-50)
	_G[scaleslide:GetName().."Text"]:SetText("Trapper Frame Scale")
	_G[scaleslide:GetName().."High"]:SetText("1.0")
	_G[scaleslide:GetName().."Low"]:SetText("0.4")
	scaleslide:SetScript("OnValueChanged",trapper_options_scale)
	
	Trapper_opts.scale=Trapper_vars.scale
	
	trapper_options:SetScript("OnEvent",trapper_options_processevent)
	InterfaceOptions_AddCategory(trapper_options)
end

function trapper_options_processevent(self,event,...)
	if event=="PLAYER_REGEN_DISABLED" then
		trapper_options:UnregisterEvent("PLAYER_REGEN_DISABLED")
		trapper_options:RegisterEvent("PLAYER_REGEN_ENABLED")
		trapper_options_disableall()
	end
	
	if event=="PLAYER_REGEN_ENABLED" then
		trapper_options:UnregisterEvent("PLAYER_REGEN_ENABLED")
		trapper_options:RegisterEvent("PLAYER_REGEN_DISABLED")
		trapper_options_enableall()
	end
end

function trapper_options_scale()
	local s=trapper_options.scaleslide:GetValue()
	
	Trapper_vars.scale=s
	trapper:SetScale(s)
end

function trapper_options_cancel()
	local s=Trapper_opts.scale
	
	Trapper_vars.scale=s
	
	trapper:SetScale(s)
	trapper_options.UnregisterAllEvents()
end

function trapper_options_ok()
	Trapper_opts.scale=Trapper_vars.scale
	trapper_options.UnregisterAllEvents()
end

function trapper_options_show()
	if UnitAffectingCombat("player") then
		trapper_options_disableall()
		trapper_options:RegisterEvent("PLAYER_REGEN_ENABLED")
	else
		trapper_options_enableall()
		trapper_options:RegisterEvent("PLAYER_REGEN_DISABLED")
	end
	
	trapper_options.scaleslide:SetValue(Trapper_vars.scale)
end

function trapper_options_disableall()
	local scaleslide=trapper_options.scaleslide
	local s=scaleslide:GetValue()
	
	scaleslide:SetMinMaxValues(s,s)
	scaleslide:SetValueStep(0)
end

function trapper_options_enableall()
	local scaleslide=trapper_options.scaleslide
	
	scaleslide:SetMinMaxValues(0.4,1.0)
	scaleslide:SetValueStep(0.05)
end

--
----------------------
-- end options stuff
----------------------
--

function trapper:new()
	local trapperbg
	
	
-- this is only for hunters
-- if you aren't a hunter, just don't do anything

	if not (UnitClass("player")=="Hunter") then
		return
	end
		
	self:SetToplevel(true)
	self:EnableMouse(true)
	self:SetMovable(true)
	self:SetWidth(100)
	self:SetHeight(66)

	self:SetBackdrop({
				bgFile="",
				edgeFile="",
				tile=false, tileSize=0, edgeSize=10,
				insets={left=0, right=0, top=0, bottom=0}
				})

-- create a texture to put in the background
-- this way we can actually adjust its size rather
-- than let wow do it

	trapperbg=self:CreateTexture(nil,"OVERLAY")
	trapperbg:SetTexture("Interface\\AddOns\\Trapper\\icons\\beartrap.tga")
	trapperbg:SetTexCoord(0,0.6640625,0,0.84375)
	trapperbg:SetPoint("TOP",self,"TOP",0,0)
	trapperbg:SetWidth(100)
	trapperbg:SetHeight(66)

	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("SPELLS_CHANGED")
	self:RegisterEvent("VARIABLES_LOADED")
	self:SetScript("OnMouseDown",self.mousedown)
	self:SetScript("OnMouseUp",self.mouseup)
	self:SetScript("OnEvent",self.processevent)
	self:SetPoint("CENTER",UIParent)
	self:Show()
end

function trapper:mousedown(button)
	if button=="RightButton" then
		trapper:StartMoving()
	end
end

function trapper:mouseup(button)
	if button=="RightButton" then
		trapper:StopMovingOrSizing()
	end
end

function trapper:processevent(event,...)
	if event=="PLAYER_LOGIN" then
		self:buildbuttons()
		DEFAULT_CHAT_FRAME:AddMessage("Trapper "..ver.." initialized.")
		DEFAULT_CHAT_FRAME:AddMessage(ntraps.." trap types enumerated.")
	end
	
-- if the player is in combat, we'll get bizarre behavior (or even errors) so lets not try
-- anything while in combat.  This might happen because SPELLS_CHANGED happens when the
-- player opens the spell book as well as when new spells come in and even when the player
-- changes weapons.  There is no way to know which of those events triggered the event.

-- the only time we really care about changing the buttons is when the player learns a new spell.
-- this is never going to happen while in combat, so we can just use UnitAffectingCombat() to
-- keep bizarre hide/show behavior from happening by doing nothing.

-- Why bother with this?  Because this will let the addon change buttons if you learn a new trap.
-- Something difficult to test and I'm not about to level up several hunters to 16 just to test it.

	if event=="SPELLS_CHANGED" then
		if not UnitAffectingCombat("player") then
			self:buildbuttons()
		end
	end
	
	if event=="VARIABLES_LOADED" then
		if Trapper_vars.scale==nil or Trapper_vars.scale==0 then Trapper_vars.scale=1.0 end

		trapper:SetScale(Trapper_vars.scale)

		-- initiate the options panel
		trapper_init_options()
	end
end

function trapper:buildbuttons()

	local t
	local W
	local H
	local offx={0,-36,36,-36,36}
	local offy={0,-18,-18,18,18}
	
-- enumerate the traps.  Setting the local global (like that term?) trap names,
-- icon paths and spell names

	ntraps=trapper:enumeratetraps()
	
-- if we have no traps, do nothing
	if ntraps == 0 then
		return
	end
		
-- set state spell attributes &
-- build statebutton string

	for t = 1, ntraps do

-- Create buttons for each trap
		if trap_b[t]==nil then
			trap_b[t] = CreateFrame("Button","trapper_button"..t,trapper,"SecureActionButtonTemplate")
			trap_b[t].cooldown = CreateFrame("Cooldown","trapper_button"..t.."Cooldown",trap_b[t],"CooldownFrameTemplate")
		end

		trap_b[t]:SetSize(32*Trapper_opts.scale,32*Trapper_opts.scale);
		
-- Set type and spell for each state
		trap_b[t]:SetAttribute("type","spell")
		trap_b[t]:SetAttribute("spell",trap_sid[t])

-- Set the image for the button
		trap_b[t]:SetNormalTexture(trap_i[t])
		trap_b[t]:SetPushedTexture(trap_i[t])

		trap_b[t]:SetPoint("CENTER",trapper,offx[t],offy[t])
		trap_b[t]:RegisterEvent("SPELL_UPDATE_COOLDOWN")
		trap_b[t]:SetScript("OnEvent",trapper_buttonevent)
		trap_b[t]:SetScript("OnEnter",trapper_tooltipon)
		trap_b[t]:SetScript("OnLeave",trapper_tooltipoff)
		
		trap_b[t].tipID=trap_sid[t]

		trap_b[t]:Show()
	end
end

function trapper_buttonevent(thisone,event,arg1,arg2,...)
	local t
	local start,dur,enable
	local name
	local cooldown

-- I have no idea why this works
-- check that the event in question is SPELL_UPDATE_COOLDOWN
	if event=="SPELL_UPDATE_COOLDOWN" then
		cooldown=_G[thisone:GetName().."Cooldown"]
		start,dur,enable=GetSpellCooldown(thisone:GetAttribute("spell"))
		CooldownFrame_SetTimer(cooldown,start,dur,enable)
	end
end

function trapper_tooltipon(self)
-- there is only one tooltip and its name is GameTooltip
	
	GameTooltip:SetOwner(self,"ANCHOR_BOTTOMLEFT")

-- set the spell name so we get the ingame spell info associated with this button
-- this is why we grabbed the spellbook id in the enumeration process and made
-- sure we had the right rank
	
	GameTooltip:SetSpellByID(self.tipID,BOOKTYPE_SPELL)
	GameTooltip:Show()
end

function trapper_tooltipoff(self)
  GameTooltip:Hide()
end

function trapper:IsSpellKnown(id)
	local s = FindSpellBookSlotBySpellID(id)
	if s and s<=MAX_SPELLS then
		local type,i = GetSpellBookItemInfo(s,BOOKTYPE_SPELL)
		return i==id and type ~='FUTURESPELL'
	end
end

function trapper:enumeratetraps()

	-- global spellIDs for the highest rank of various traps [freezing,frost,explosive,immolation,snake]
	local trap_spellIDs	={1499,13809,13813,13795,34600}
	local trap_dropIDs  ={1499,13809,13813,13795,34600}

	-- iterate through all the possible traps to add them to the trap list
	local a=0
	local t
	local r

	for t = 1, 5 do
		local gname
		local name
		local icon

		if trapper:IsSpellKnown(trap_spellIDs[t]) then
			a=a+1
			trap_n[a],r,trap_i[a]=GetSpellInfo(trap_spellIDs[t])
			trap_sid[a]=trap_spellIDs[t]
			trap_n[a]=trap_n[a]..(r or nil)
		end
	end
	-- return the number of traps we counted
	return a
end

trapper:new()
